package com.jtauber.fop.apps;

// FOP
import com.jtauber.fop.fo.FOTreeBuilder;

// SAX
import org.xml.sax.Parser;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

// Java
import java.io.PrintWriter;
import java.io.IOException;

/**
 * A FOProcessor that takes a SAX Parser and InputSource and parses the incoming representing of the formatting object tree
 * 
 * Instantiate this class with a SAX Parser and InputSource and then run the format method with a PrintWriter the PDF is to be given to
 */
public class SAXProcessor extends FOProcessor {

	private Parser parser;
	private InputSource source;
	
/**
 * create a SAX-driven FOProcessor with the given Parser operating on the given InputSource
 */
public SAXProcessor(Parser parser, InputSource source) {
	super(); // setup the FOProcessor
	this.parser = parser;
	this.source = source;
}
/**
 * Format the document to PDF on the given PrintWriter
 *
 */
public void format(PrintWriter writer) throws FOPException {
	this.parser.setDocumentHandler(this.treeBuilder);
	try {
		this.parser.parse(this.source);
	} catch (SAXException e) {
		if (e.getException() instanceof FOPException)
			throw (FOPException)e.getException();
		else
			throw new FOPException(e.getMessage());
	} catch (IOException e) {
		throw new FOPException(e.getMessage());
	}
	layoutput(writer);
}
}