package com.jtauber.fop.fo;

import com.jtauber.fop.layout.Document;
import com.jtauber.fop.apps.FOPException;

import com.jtauber.fop.fo.flow.*;
import com.jtauber.fop.fo.pagination.*;

import com.jclark.xml.sax.Driver;

import org.xml.sax.HandlerBase;
import org.xml.sax.SAXException;
import org.xml.sax.InputSource;
import org.xml.sax.AttributeList;


import java.util.Hashtable;
import java.io.IOException;


public class FOTreeBuilder extends HandlerBase {

	private Hashtable fobjTable;

	private PropertyListBuilder propertyListBuilder;
	
	FObj currentFObj;
	public FObj rootFObj;

	Document doc;
	
	public FOTreeBuilder(Document doc) {

		this.currentFObj = null;
		this.rootFObj = null;

		this.propertyListBuilder = new PropertyListBuilder();
		
		this.fobjTable = new Hashtable();
		
		this.doc = doc;

		this.fobjTable.put("fo:root", Root.maker());
		this.fobjTable.put("fo:layout-master-set", LayoutMasterSet.maker());
		this.fobjTable.put("fo:simple-page-master", SimplePageMaster.maker());
		this.fobjTable.put("fo:region-body", RegionBody.maker());
		this.fobjTable.put("fo:region-before", RegionBefore.maker());
		this.fobjTable.put("fo:region-after", RegionAfter.maker());
		this.fobjTable.put("fo:page-sequence", PageSequence.maker());
		this.fobjTable.put("fo:sequence-specification", SequenceSpecification.maker());
		this.fobjTable.put("fo:sequence-specifier-single", SequenceSpecifierSingle.maker());
		this.fobjTable.put("fo:sequence-specifier-repeating", SequenceSpecifierRepeating.maker());
		this.fobjTable.put("fo:sequence-specifier-alternating", SequenceSpecifierAlternating.maker());
		this.fobjTable.put("fo:flow", Flow.maker());
		this.fobjTable.put("fo:static-content", StaticContent.maker());
		this.fobjTable.put("fo:block", Block.maker());
		this.fobjTable.put("fo:list-block", ListBlock.maker());
		this.fobjTable.put("fo:list-item", ListItem.maker());
		this.fobjTable.put("fo:list-item-label", ListItemLabel.maker());
		this.fobjTable.put("fo:list-item-body", ListItemBody.maker());
		this.fobjTable.put("fo:page-number", PageNumber.maker());
		this.fobjTable.put("fo:display-sequence", DisplaySequence.maker());
		this.fobjTable.put("fo:inline-sequence", InlineSequence.maker());
	}
	public void characters(char data[], int start, int length) {
		currentFObj.addCharacters(this.doc,data,start,start+length);
	}
	public void endElement(String name) {
		currentFObj.end(this.doc);
		currentFObj = (FObj)currentFObj.getParent();
	}
public void go() throws FOPException {
	System.err.println("constructing areas/spaces from formatting objects");
	this.rootFObj.layout();
}
public void startDocument() {
	System.err.println("building fully-computed formatting object tree");
}
	public void startElement(String name, AttributeList attlist) throws SAXException {
		FObj fobj;
		FObj.Maker fobjMaker;

		fobjMaker = (FObj.Maker)fobjTable.get(name);

		if (fobjMaker == null) {
			//System.err.println("WARNING: Unknown formatting object "+name);
			fobjMaker = new FObj.Maker();
		}

		try {
			fobj = fobjMaker.make(this.doc,
				currentFObj, this.propertyListBuilder.makeList(attlist, (currentFObj == null) ? null : currentFObj.properties));
		} catch (FOPException e) {
			throw new SAXException(e);
		}

		if (rootFObj == null) {
			rootFObj = fobj;
			if (!fobj.foName.equals("fo:root")) {
				throw new SAXException(new FOPException("Root element must be fo:root, not "+fobj.foName));
			}
		}
		else {
			currentFObj.addChild(fobj);
		}
		
		currentFObj = fobj;
	}
}