package com.jtauber.fop.layout;

import java.io.PrintWriter;
import java.util.Vector;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Stack;
import com.jtauber.fop.fo.flow.StaticContent;

import com.jtauber.fop.apps.FOPException;				   

public class Document {

	Layout layout; // the output-dependent layout code interface
	FontInfo fontInfo; // the object that provides output-independent info on fonts
	
	FontState defaultFontState;

	PageMasterFactory pageMasterFactory;

	Vector pageList; // list of all the pages
	Page currentPage;
	Area currentArea;

	StaticContent staticBefore;
	StaticContent staticAfter;

	private int currentPageNumber;
	private boolean currentAreaContainerIsStatic;

	private boolean inList = false;
	private boolean inListLabel = false;
	private boolean inListBody = false;
	private int distanceBetweenStarts;
	private int labelSeparation;
	public Document(Layout layout) {

		this.layout = layout;
		this.fontInfo = new FontInfo();
		layout.setupFontInfo(this.fontInfo);
		
		this.currentPageNumber = 1; // this shouldn't be here
		this.pageList = new Vector();
	}
	public void addPage(Page p) {
		pageList.addElement(p);
	}
	public void addText(FontState fs, char[] ca, int ts, int te) throws FOPException {
		if (!(currentArea instanceof BlockArea))
			throw new FOPException("text encountered outside block");
		int start = ((BlockArea)currentArea).addText(fs,ca,ts,te);
		if (start != -1) {
			Stack blockAreaStack = new Stack();
			while (currentArea instanceof BlockArea) {
				blockAreaStack.push(currentArea);
				endBlockArea(0);
			}
			if (this.currentAreaContainerIsStatic) {
				System.err.println("WARNING: static-content was truncated");
			} else {
				endPage();
				startPage();
				while (!blockAreaStack.empty()) {
					startBlockAreaDuplicate((BlockArea)blockAreaStack.pop());
				}
				addText(fs,ca,start,te);
			}
		}
	}
	public void addText(FontState fs, String text) throws FOPException {
		char[] ca = text.toCharArray();
		int ts = 0;
		int te = text.length();
		this.addText(fs,ca,ts,te);
	}
	public void endBlockArea(int space_after) throws FOPException {
		if (!(currentArea instanceof BlockArea)) {
			throw new FOPException("Attempting to end block area when none have been started. This is a bug in FOP that I'm working on");
		}
		currentArea.end();
		currentArea.addDisplaySpace(space_after);
		currentArea.parent.increaseHeight(currentArea.getHeight());
		currentArea = currentArea.parent;
		if (currentArea instanceof BlockArea)
			currentArea.start();
	}
	public void endListBlockArea(int space_after) {
		currentArea.end();
		if (space_after != 0) {
			currentArea.addDisplaySpace(space_after);
		}
		currentArea.parent.increaseHeight(currentArea.getHeight());
		this.inList = false;
		currentArea = currentArea.parent;
		if (currentArea instanceof BlockArea)
			currentArea.start();
	}
	public void endPage() {
			System.err.println("end page " + this.currentPageNumber);
			this.addPage(currentPage);
			this.currentPageNumber++;
	}
	public FontInfo getFontInfo() {
		return this.fontInfo;
	}
	public String getPageNumber() {
		return Integer.toString(this.currentPageNumber);
	}
public Vector getPages() {
	return this.pageList;
}
	public void setDefaultFontState(FontState defaultFontState) {
		this.defaultFontState = defaultFontState;
	}
public void setInListBody(boolean in) {
	this.inListBody = in;
}
public void setInListLabel(boolean in) {
	this.inListLabel = in;
}
	public void setPageMasterFactory(PageMasterFactory pageMasterFactory) {
		this.pageMasterFactory = pageMasterFactory;
	}
	public void setStaticAfter(StaticContent sc) {
		this.staticAfter = sc;
	}
	public void setStaticBefore(StaticContent sc) {
		this.staticBefore = sc;
	}
	public void startBlockArea(FontState fontState, int align, int alignLastLine, int lineHeight, int startIndent, int endIndent, int spaceBefore) {
		if (currentArea instanceof BlockArea)
			currentArea.end();
		if (spaceBefore != 0)
			currentArea.addDisplaySpace(spaceBefore);
		boolean topAlign = false;
		if (this.inListLabel) {
			endIndent = endIndent + (currentArea.getWidth() - this.distanceBetweenStarts) + labelSeparation;
			topAlign = true;
		}
		if (this.inListBody) {
			startIndent = startIndent + this.distanceBetweenStarts;
		}
		BlockArea ba = new BlockArea(fontState,	currentArea.getWidth(), currentArea.spaceLeft(), startIndent, endIndent, align, alignLastLine, lineHeight, topAlign);
		ba.start();
		currentArea.addChild(ba);
		currentArea = ba;
	}
	public void startBlockAreaDuplicate(BlockArea ba) {
		startBlockArea(ba.fontState,ba.align,ba.alignLastLine,ba.lineHeight,ba.startIndent,ba.endIndent,0);
	}
	public void startListBlockArea(FontState fontState, int startIndent, int endIndent, int spaceBefore, int distanceBetweenStarts, int labelSeparation, int spaceBetweenRows) {
		this.inList = true;
		this.distanceBetweenStarts = distanceBetweenStarts;
		this.labelSeparation = labelSeparation;
		
		if (currentArea instanceof BlockArea)
			currentArea.end();
		if (spaceBefore != 0)
			currentArea.addDisplaySpace(spaceBefore);

	    // note that lineHeight = 0 (bad, but what can I do?)
		BlockArea ba = new BlockArea(fontState,	currentArea.getWidth(), currentArea.spaceLeft(), startIndent, endIndent, 0, 0, 0, false);
		ba.start();
		currentArea.addChild(ba);
		currentArea = ba;
	}
	public void startPage() throws FOPException {
		// GET PAGE MASTER
		PageMaster pageMaster = this.pageMasterFactory.getNextPageMaster();
		while (pageMaster == null) {
			// System.err.println("moving on to next sequence specifier");
			this.pageMasterFactory = this.pageMasterFactory.getNext();
			if (this.pageMasterFactory == null) {
				throw new FOPException("out of sequence specifiers (subsequent versions of FOP will allow this)");
			}
			pageMaster = this.pageMasterFactory.getNextPageMaster();
		}

		// MAKE A NEW PAGE
		
		currentPage = pageMaster.makePage();
		
		if (this.staticBefore != null) {
			if (currentPage.getBefore() != null) {
				this.currentAreaContainerIsStatic = true;
				currentArea = currentPage.getBefore();
				this.staticBefore.layout();
			}
		}
		
		if (this.staticAfter != null) {
			if (currentPage.getAfter() != null) {
				this.currentAreaContainerIsStatic = true;
				currentArea = currentPage.getAfter();
				this.staticAfter.layout();
			}
		}

		this.currentAreaContainerIsStatic = false;
		currentArea = currentPage.getBody();
	}
}