package com.jtauber.fop.layout;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.Enumeration;

import com.jtauber.pdf.*;

public class XMLLayout implements Layout {
	String xml;
	int indent;
	public XMLLayout(String producer) {
		this.xml = "<?xml version=\"1.0\"?>\n<!-- produced by " + producer + " -->\n";
		this.indent = 0;
	}
private void doBoth(String tag) {
	doIndent();
	xml += tag + "\n";
}
private void doEmpty(String tag) {
	doIndent();
	xml += tag + "\n";
}
private void doEnd(String tag) {
	indent--;
	doIndent();
	xml += tag + "\n";
}
private void doIndent() {
	String s = "";
	for (int i= 0; i<this.indent; i++) {
		s += "  ";
	}
	xml += s;
}
private void doStart(String tag) {
	doIndent();
	xml += tag + "\n";
	indent++;
}
public void go(Document doc, PrintWriter writer) throws IOException {
	System.err.println("converting areas/spaces to XML");
	Enumeration e = doc.getPages().elements();
	while (e.hasMoreElements()) {
		this.makePage((Page)e.nextElement());
	}
	System.err.println("writing out XML");
	writer.write(this.xml);
	writer.flush();
}
	public void makeAreaContainer(AreaContainer areaContainer) {
		doStart("<AreaContainer>");

		Enumeration e = areaContainer.getChildren().elements();
		while (e.hasMoreElements()) {
			Box b = (Box)e.nextElement();
			if (b instanceof BlockArea)
				makeBlockArea((BlockArea)b);
			else if (b instanceof DisplaySpace)
				makeDisplaySpace((DisplaySpace)b);
			else
				System.err.println("Area container had something other than BlockArea and DisplaySpace");
		}

		doEnd("</AreaContainer>");
	}
	public void makeBlockArea(BlockArea blockArea) {
		doStart("<BlockArea>");
		
		Enumeration e = blockArea.getChildren().elements();
		while (e.hasMoreElements()) {
			Box b = (Box)e.nextElement();
			if (b instanceof LineArea)
				makeLineArea((LineArea)b);
			else if (b instanceof DisplaySpace)
				makeDisplaySpace((DisplaySpace)b);
			else if (b instanceof BlockArea)
				makeBlockArea((BlockArea)b);
			else
				System.err.println("Block area had something other than LineArea, BlockArea or DisplaySpace");
		}

		doEnd("</BlockArea>");
	}
	public void makeDisplaySpace(DisplaySpace displaySpace) {
		doEmpty("<DisplaySpace size=\"" + displaySpace.getSize() +"\"/>");
	}
	public void makeInlineArea(InlineArea inlineArea) {
		doBoth("<InlineArea>"+ inlineArea.getText() + "</InlineArea>");
	}
	public void makeInlineSpace(InlineSpace inlineSpace) {
		doEmpty("<InlineSpace/>");
	}
	public void makeLineArea(LineArea lineArea) {
		doStart("<LineArea>");
		
		Enumeration e = lineArea.getChildren().elements();
		while (e.hasMoreElements()) {
			Box b = (Box)e.nextElement();
			if (b instanceof InlineArea)
				makeInlineArea((InlineArea)b);
			else if (b instanceof InlineSpace)
				makeInlineSpace((InlineSpace)b);
			else
				System.err.println("Line area had something other than InlineArea and InlineSpace");
		}

		doEnd("</LineArea>");
	}
	 public void makePage(Page page) {
		AreaContainer body, before, after;
		doStart("<Page>");
		body = page.getBody();
		before = page.getBefore();
		after = page.getAfter();

		if (before != null)
			makeAreaContainer(before);
		
		makeAreaContainer(body);

		if (after != null)
			makeAreaContainer(after);

		doEnd("</Page>");
	}
public void setupFontInfo(FontInfo fontInfo) {
	com.jtauber.fop.layout.pdf.FontSetup.setup(fontInfo); // so we have the PDF font metrics
}
}